using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;

public class LiquidMotion
{
    private MVRScript _root;
    private Transform _BottleRoot;

    protected JSONStorableBool IsActive;
    protected JSONStorableStringChooser _personSelectJSON;
    protected UIDynamicPopup _personDisplayPopup;
    protected JSONStorableFloat _motionFrame;
    protected List<String> _atomBodyChoices;
    protected Atom _selectAtom;
    private Rigidbody _mouthTrigger;
    private Rigidbody _lipTrigger;
    private FreeControllerV3 _thisControl;

    private float _lerp_value = 0f;

    public void Init(MVRScript root, Atom thisAtom)
    {
        _root = root;

        _thisControl = thisAtom.freeControllers.First(fc => fc.name == "control");

        IsActive = new JSONStorableBool("Active", false, SetTargetActive);
        _root.RegisterBool(IsActive);
        UIDynamicToggle toggle2 = _root.CreateToggle(IsActive, false);
        toggle2.label = "Active Drink Motion";
        toggle2.height = 60;

        var btn = _root.CreateButton("Person Update", false);

        btn.button.onClick.AddListener(() =>
        {
            UpdateAtomData();
        });

        _personSelectJSON = new JSONStorableStringChooser("Target Choice", null, null, "", OnTargetPersonSelect);
        
        _personSelectJSON.storeType = JSONStorableParam.StoreType.Physical;
        _root.RegisterStringChooser(_personSelectJSON);

        _personDisplayPopup = _root.CreateScrollablePopup(_personSelectJSON, false);
        _personDisplayPopup.popupPanelHeight = 300f;
        _personDisplayPopup.popup.onOpenPopupHandlers += SyncPersonChoices;
        _personDisplayPopup.label = "Person:";

        _motionFrame = new JSONStorableFloat("Motion Length", 0.35f, OnMotionFrameChanged, 0f, 1f);
        _root.RegisterFloat(_motionFrame);
        _root.CreateSlider(_motionFrame, false);

        UpdateAtomData();
    }

    public void SetBottleRoot(Transform bottle_root)
    {
        _BottleRoot = bottle_root;
    }

    private void UpdateAtomData()
    {
        _atomBodyChoices = GetAtomListChoices();
        _personSelectJSON.choices = _atomBodyChoices;
    }

    protected List<string> GetAtomListChoices()
    {
        List<string> atomChoices = new List<string>();
        Atom atom;
        foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
        {
            atom = SuperController.singleton.GetAtomByUid(atomUID);
            JSONStorable geometry = atom.GetStorableByID("geometry");
            if (geometry != null)
            {
                atomChoices.Add(atomUID);
                //SuperController.LogMessage(">>" + atomUID);
            }
        }
        return atomChoices;
    }

    private void SetTargetActive(JSONStorableBool jb)
    {
        if (!jb.val)
        {


        }
    }

    public void MotionFixedUpdate()
    {
        if (IsActive.val)
        {
            try
            {
                Vector3 targetPosition;
                Vector3 targetUpDirection;
                Transform tempTransform;
                Quaternion targetRotation;
                //Quaternion targetRotationInverse;

                targetPosition = _lipTrigger.transform.position;
                targetUpDirection = _lipTrigger.transform.up;
                targetRotation = _lipTrigger.transform.rotation;
                //targetRotationInverse = _lipTrigger.transform.rotation;

                float motion_frame = _motionFrame.val;
                _lerp_value += 0.01f;
                if (_lerp_value > 1f) _lerp_value = 1f;

                // -0.06f
                float height = motion_frame * 0.4f * -1f - 0.05f;
                Vector3 offsetPosition = new Vector3(-0.07f, height, (motion_frame * 0.02f) + 0.225f);

                tempTransform = _thisControl.transform;
                targetPosition += offsetPosition;
                _thisControl.transform.position = targetPosition;

                tempTransform.rotation = targetRotation;
                //float dampSpeed = 0.1f;
                //Vector3 nPosition = Vector3.Lerp(tempTransform.position, offsetPosition, 1f);
                //Vector3 n_p = offsetPosition - tempTransform.position;
                //_thisControl.transform.Translate(offsetPosition);
                //_thisControl.transform.position = Vector3.Lerp(tempTransform.position, offsetPosition, 1f);
                // -80f
                //float angle = -80f + (90f - (90f * (1f - motion_frame)));
                //_thisControl.transform.Rotate(angle, 0f, 0f);

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }
    }

    protected void OnTargetPersonSelect(string target_name)
    {

        _selectAtom = SuperController.singleton.GetAtomByUid(target_name);
        _mouthTrigger = _selectAtom.rigidbodies.First(rb => rb.name == "MouthTrigger");
        _lipTrigger = _selectAtom.rigidbodies.First(rb => rb.name == "LipTrigger");

    }

    protected void SyncPersonChoices()
    {
        SuperController.LogMessage(">> SyncPersonChoices ");
        //UpdateOrificeChoices();

        //_mouthTrigger = _atom1.rigidbodies.First(rb => rb.name == "MouthTrigger");
    }

    protected void OnMotionFrameChanged(float value)
    {
        _lerp_value = 0f;
    }

}