using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

public class LiquidOpt
{
    private MVRScript _root;
    private Transform _LiquidRoot;

    private float CurrScale = 1f;

    private JSONStorableFloat LiquidHeight;
    private JSONStorableFloat LiquidAlpha;
    private JSONStorableColor LiquidColor;
    private JSONStorableColor LiquidTopColor;
    private JSONStorableColor FoamColor;
    private JSONStorableFloat FoamHeight;
    private JSONStorableFloat EmissionValue;

    private List<String> _orificeFemaleChoices;
    private JSONStorableStringChooser _orificeJSON;
    private UIDynamicPopup _displayPopup; // any UI element, just to check if visible

    private bool _show_color_option = false;
    private string _curr_orifice_type = "";

    public void Init(MVRScript root)
    {
        _root = root;
        JSONStorableString helpText = new JSONStorableString("Caution",
                    "<color=#000><size=30><b>LIQUID CUSTOM</b></size></color>");
        UIDynamic helpTextfield = _root.CreateTextField(helpText, false);
        helpTextfield.height = 20.0f;
        //**************  Liquid
        LiquidHeight = new JSONStorableFloat("Liquid Height", 0.5f, OnLiquidHeightChanged, 0f, 1f);
        _root.RegisterFloat(LiquidHeight);
        _root.CreateSlider(LiquidHeight, false);

        //**************  Liquid Alpha
        LiquidAlpha = new JSONStorableFloat("Liquid Alpha", 0.5f, OnLiquidAlphaChanged, 0f, 1f);
        _root.RegisterFloat(LiquidAlpha);
        _root.CreateSlider(LiquidAlpha, false);

        //**************  Foam Height
        FoamHeight = new JSONStorableFloat("Foam Height", 0f, OnFoamHeightChanged, 0f, 1f);
        _root.RegisterFloat(FoamHeight);
        _root.CreateSlider(FoamHeight, false);

        EmissionValue = new JSONStorableFloat("Emission Value", 0.5f, OnEmissionValueChanged, 0.5f, 4f);
        _root.RegisterFloat(EmissionValue);
        _root.CreateSlider(EmissionValue, false);

        HSVColor hsvc1 = HSVColorPicker.RGBToHSV(0.5f, 0.5f, 1f);
        LiquidColor = new JSONStorableColor("Liquid Color", hsvc1, LiquidColorCallback);
        _root.RegisterColor(LiquidColor);
        
        HSVColor hsvc2 = HSVColorPicker.RGBToHSV(0.6f, 0.7f, 1f);
        LiquidTopColor = new JSONStorableColor("Liquid Top Color", hsvc2, LiquidTopColorCallback);
        _root.RegisterColor(LiquidTopColor);
        
        HSVColor hsvc3 = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
        FoamColor = new JSONStorableColor("Foam Color", hsvc3, FoamColorCallback);
        _root.RegisterColor(FoamColor);
        
        _orificeFemaleChoices = new List<string>();
        _orificeFemaleChoices.Add("Custom");
        _orificeFemaleChoices.Add("Beer");
        _orificeFemaleChoices.Add("Wine1");
        _orificeFemaleChoices.Add("Wine2");
        _orificeFemaleChoices.Add("Whiskey");
        _orificeFemaleChoices.Add("Water");
        _orificeFemaleChoices.Add("Milk");
        _orificeFemaleChoices.Add("GreenJuice");
        //Milk

        _orificeJSON = new JSONStorableStringChooser("OrificeChoice", null, "Custom", "", OrificeSelect);
        _orificeJSON.storeType = JSONStorableParam.StoreType.Physical;
        _root.RegisterStringChooser(_orificeJSON);

        _displayPopup = _root.CreateScrollablePopup(_orificeJSON, false);
        _displayPopup.popupPanelHeight = 400f;
        _displayPopup.popup.onOpenPopupHandlers += SyncOrificeChoices;
        _displayPopup.label = "Liquid Type:";

        _orificeJSON.choices = _orificeFemaleChoices;



        var btn = _root.CreateButton("Liquid Color Option", false);
        btn.button.onClick.AddListener(() =>
        {
            ShowLiquidColorOpt(!_show_color_option);
        });

    }

    public void ShowLiquidColorOpt(bool is_show)
    {
        _show_color_option = is_show;
        if (is_show)
        {
            _root.CreateColorPicker(LiquidColor, false);
            _root.CreateColorPicker(LiquidTopColor, false);
            _root.CreateColorPicker(FoamColor, false);
        }
        else
        {
            _root.RemoveColorPicker(LiquidColor);
            _root.RemoveColorPicker(LiquidTopColor);
            _root.RemoveColorPicker(FoamColor);
        }
    }

    protected void OrificeSelect(string orificeType)
    {
        if (orificeType == "Custom") return;
        if (_curr_orifice_type == orificeType) return;

        switch (orificeType)
        {
            case "Beer":
                LiquidAlpha.val = 0.8f;
                LiquidColor.val = HSVColorPicker.RGBToHSV(0.56f, 0.262f, 0f);
                FoamHeight.val = 0.16f;
                LiquidTopColor.val = HSVColorPicker.RGBToHSV(0.792f, 0.517f, 0.274f);
                FoamColor.val = HSVColorPicker.RGBToHSV(0.792f, 0.517f, 0.274f);
                break;
            case "Wine1":
                LiquidAlpha.val = 0.5f;
                LiquidColor.val = HSVColorPicker.RGBToHSV(0.698f, 0.015f, 0.69f);
                FoamHeight.val = 0f;
                LiquidTopColor.val = HSVColorPicker.RGBToHSV(0.835f, 0.227f, 0.831f);
                FoamColor.val = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
                break;
            case "Wine2":
                LiquidAlpha.val = 0.8f;
                LiquidColor.val = HSVColorPicker.RGBToHSV(0.403f, 0f, 0f);
                FoamHeight.val = 0f;
                LiquidTopColor.val = HSVColorPicker.RGBToHSV(0.705f, 0.168f, 0.168f);
                FoamColor.val = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
                break;
            case "Whiskey":
                LiquidAlpha.val = 0.5f;
                LiquidColor.val = HSVColorPicker.RGBToHSV(0.431f, 0.204f, 0.0353f);
                FoamHeight.val = 0f;
                LiquidTopColor.val = HSVColorPicker.RGBToHSV(0.564f, 0.317f, 0.137f);
                FoamColor.val = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
                break;
            case "Water":
                LiquidAlpha.val = 0.5f;
                LiquidColor.val = HSVColorPicker.RGBToHSV(0.5f, 0.5f, 1f);
                FoamHeight.val = 0f;
                LiquidTopColor.val = HSVColorPicker.RGBToHSV(0.6f, 0.7f, 1f);
                FoamColor.val = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
                break;
            case "Milk":
                LiquidAlpha.val = 0.8f;
                LiquidColor.val = HSVColorPicker.RGBToHSV(0.776f, 0.776f, 0.776f);
                FoamHeight.val = 0f;
                LiquidTopColor.val = HSVColorPicker.RGBToHSV(0.8f, 0.8f, 0.8f);
                FoamColor.val = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
                break;
            case "GreenJuice":
                LiquidAlpha.val = 0.8f;
                LiquidColor.val = HSVColorPicker.RGBToHSV(0.215f, 0.333f, 0.015f);
                FoamHeight.val = 0f;
                LiquidTopColor.val = HSVColorPicker.RGBToHSV(0.266f, 0.380f, 0.015f);
                FoamColor.val = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
                break;
            default:
                
                break;
        }

        //SuperController.LogMessage(">> OrificeSelect " + orificeType);
        UpdateValue();
        _curr_orifice_type = orificeType;
        _orificeJSON.val = orificeType;
        
    }

    protected void SyncOrificeChoices()
    {
        //SuperController.LogMessage(">> SyncOrificeChoices ");
        //UpdateOrificeChoices();
    }

    public void SetLiquidRoot(Transform liquid_root)
    {
        _LiquidRoot = liquid_root;
    }

    public void UpdateCurrScale(float value)
    {
        //SuperController.LogMessage(">> UpdateCurrScale");
        CurrScale = value;
        OnLiquidHeightChanged(LiquidHeight.val);
    }

    public void SetLiquidType()
    {

    }

    public void UpdateValue()
    {
        //SuperController.LogMessage(">> UpdateValue");
        OnLiquidHeightChanged(LiquidHeight.val);
        FoamColorCallback(FoamColor);
        LiquidTopColorCallback(LiquidTopColor);
        LiquidColorCallback(LiquidColor);
        OnLiquidAlphaChanged(LiquidAlpha.val);
        OnFoamHeightChanged(FoamHeight.val);
        OnEmissionValueChanged(EmissionValue.val);
        //SuperController.LogMessage(">> UpdateValue " + FoamHeight.val);
    }

    public void UpdateHeight()
    {
        OnLiquidHeightChanged(LiquidHeight.val);
    }

    void OnLiquidHeightChanged(float value)
    {
        //SuperController.LogMessage(">>" + _LiquidRoot);
        if (_LiquidRoot == null) return;

        Renderer renderer = _LiquidRoot.GetComponent<Renderer>();
        float v_value = value * CurrScale;

        float s_min = 0.5f - (((0.5f - LiquidType.MinFill) * CurrScale));
        float height = s_min - ((LiquidType.MaxFill * CurrScale) * value);

        renderer.material.SetFloat("_FillAmount", height);
        renderer.material.SetFloat("_RimPower", 2f);

        _orificeJSON.val = "Custom";
    }

    // Liquid Alpha
    void OnLiquidAlphaChanged(float value)
    {
        if (_LiquidRoot == null) return;

        Renderer renderer = _LiquidRoot.GetComponent<Renderer>();
        Color cc = renderer.material.GetColor("_Colour");
        Color ct = renderer.material.GetColor("_TopColor");
        cc.a = value;
        ct.a = value;
        //SuperController.LogMessage(">>" + cc);
        renderer.material.SetColor("_Colour", cc);
        Color rc = renderer.material.GetColor("_Colour");
        rc.a = 1f;
        renderer.material.SetColor("_RimColor", rc);
        renderer.material.SetColor("_TopColor", ct);
        renderer.material.SetFloat("_RimPower", 2f);

        _orificeJSON.val = "Custom";
    }

    // Foam Height
    void OnFoamHeightChanged(float value)
    {
        if (_LiquidRoot == null) return;

        Renderer renderer = _LiquidRoot.GetComponent<Renderer>();
        float v_value = value * CurrScale;
        float height = (0.1f * value) * CurrScale;

        renderer.material.SetFloat("_Rim", height);
        _orificeJSON.val = "Custom";
    }

    // Emission Value
    void OnEmissionValueChanged(float value)
    {
        if (_LiquidRoot == null) return;

        Renderer renderer = _LiquidRoot.GetComponent<Renderer>();

        renderer.material.SetFloat("_Emission", value);

    }

    //  Liquid color
    void LiquidColorCallback(JSONStorableColor dc)
    {
        if (_LiquidRoot == null) return;

        Color c = Color.HSVToRGB(dc.valNoCallback.H, dc.valNoCallback.S, dc.valNoCallback.V);

        Renderer renderer = _LiquidRoot.GetComponent<Renderer>();
        Color cc = renderer.material.GetColor("_Colour");
        Color rc = renderer.material.GetColor("_Colour");
        Color fc = renderer.material.GetColor("_FoamColor");
        rc.a = 1f;
        c.a = cc.a;
        fc.a = cc.a;
        renderer.material.SetColor("_Colour", c);
        renderer.material.SetColor("_RimColor", rc);
        //renderer.material.SetColor("_TopColor", ct);
        renderer.material.SetColor("_FoamColor", fc);

        _orificeJSON.val = "Custom";
    }

    //  Liquid top color
    protected void LiquidTopColorCallback(JSONStorableColor dc)
    {
        if (_LiquidRoot == null) return;

        Color c = Color.HSVToRGB(dc.valNoCallback.H, dc.valNoCallback.S, dc.valNoCallback.V);
        Renderer renderer = _LiquidRoot.GetComponent<Renderer>();
        Color cc = renderer.material.GetColor("_TopColor");
        c.a = cc.a;
        renderer.material.SetColor("_TopColor", c);

        _orificeJSON.val = "Custom";
    }

    //  foam color
    protected void FoamColorCallback(JSONStorableColor dc)
    {
        if (_LiquidRoot == null) return;

        Color c = Color.HSVToRGB(dc.valNoCallback.H, dc.valNoCallback.S, dc.valNoCallback.V);
        Renderer renderer = _LiquidRoot.GetComponent<Renderer>();
        Color cc = renderer.material.GetColor("_Colour");
        c.a = cc.a;
        renderer.material.SetColor("_FoamColor", c);

        _orificeJSON.val = "Custom";
    }

}